/*
   ; Braitenberg test
   ; ================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 3 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it publishes an "IR sensor" association.
   ;                    Process 2: it implements a Braitengerg vehicle 3 obstacle
   ;                                       avoidance algorithm.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1999
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

#define		KNBSENSORS	15
#define		KNBMOTORS	2
#define		KSCALING	400
#define		KSPEED		20

static int32 vIDProcess[3];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process publishes an association between "IR sens" and a Ptr
 * on the I.R. values.
 *
 */

static void
process_1 ()
{
  int32 status;
  IRSENSOR *sensor;

  sensor = sens_get_pointer ();
  status = tim_define_association ("IR sens", (uint32 *) sensor);
  if (status < 0)
    {
      printf ("It is impossible to generate an IR sens association\r\n");
      exit (0);
    }

  exit (0);
}

/*
 * Process 2
 * ^^^^^^^^^
 *
 * This process implements the Braitenberg vehicle 3 algorithm.
 *
 */

static void
process_2 ()
{
  int32 status;
  IRSENSOR *sensor;
  uint32 sensBuf[KNBSENSORS];
  int i, s, m;
  int32 potential[KNBMOTORS], speed[KNBMOTORS];
  int32 matrix[KNBMOTORS][KNBSENSORS] =
  {
    {-5, -15, -18, 6, 4, 4, 3, 5},
    {4, 4, 6, -18, -15, -5, 5, 3}};

  mot_config_speed_1m (0, 3800, 800, 100);
  mot_config_speed_1m (1, 3800, 800, 100);

/* Wait for the IR sens association */

  do
    {
      status = tim_find_association ("IR sens");
    }
  while (status < 0);

  sensor = (IRSENSOR *) status;

/* Calculate the neuron (potentials x Gain) and add. the initial KSPEED */

  for (;;)
    {
      for (i = 0; i < KNBSENSORS; i++)
	sensBuf[i] = sensor->oProximitySensor[i];
      for (m = 0; m < KNBMOTORS; m++)
	{
	  potential[m] = 0;
	  for (s = 0; s < KNBSENSORS; s++)
	    {
	      potential[m] += (matrix[m][s] * (int32) sensBuf[s]);
	    }
	  speed[m] = (potential[m] / KSCALING) + KSPEED;
	}
      mot_new_speed_2m (speed[0], speed[1]);
    }
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched tasks are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";
  static char prName_2[] = "User 2 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();
  sens_reset ();
  str_reset ();
  mot_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  status = install_task (prName_2, 800, process_2);
  if (status == -1)
    exit (0);
  vIDProcess[2] = (uint32) status;

 return 0;
}
